package com.bmlaurus.alfresco.content;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.net.FileNameMap;
import java.net.URLConnection;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.util.logging.Logger;

import org.alfresco.webservice.authoring.AuthoringFault;
import org.alfresco.webservice.authoring.AuthoringServiceSoapBindingStub;
import org.alfresco.webservice.authoring.CheckoutResult;
import org.alfresco.webservice.content.Content;
import org.alfresco.webservice.content.ContentServiceSoapBindingStub;
import org.alfresco.webservice.repository.QueryResult;
import org.alfresco.webservice.repository.RepositoryFault;
import org.alfresco.webservice.repository.RepositoryServiceSoapBindingStub;
import org.alfresco.webservice.repository.UpdateResult;
import org.alfresco.webservice.types.CML;
import org.alfresco.webservice.types.CMLAddAspect;
import org.alfresco.webservice.types.CMLCreate;
import org.alfresco.webservice.types.CMLDelete;
import org.alfresco.webservice.types.CMLUpdate;
import org.alfresco.webservice.types.ContentFormat;
import org.alfresco.webservice.types.NamedValue;
import org.alfresco.webservice.types.Node;
import org.alfresco.webservice.types.ParentReference;
import org.alfresco.webservice.types.Predicate;
import org.alfresco.webservice.types.Query;
import org.alfresco.webservice.types.Reference;
import org.alfresco.webservice.types.ResultSet;
import org.alfresco.webservice.types.ResultSetRow;
import org.alfresco.webservice.types.Store;
import org.alfresco.webservice.types.Version;
import org.alfresco.webservice.types.VersionHistory;
import org.alfresco.webservice.util.AuthenticationUtils;
import org.alfresco.webservice.util.Constants;
import org.alfresco.webservice.util.ContentUtils;
import org.alfresco.webservice.util.ISO9075;
import org.alfresco.webservice.util.Utils;
import org.alfresco.webservice.util.WebServiceFactory;

import com.bmlaurus.alfresco.AlfrescoFile;
import com.bmlaurus.alfresco.Aspect;

/**
 * @author Antonio Caaveral
 * @version 1.0
 * @company BMLaurus
 * @email info@bmlaurus.com
 * */
public class AlfrescoContentService {

  protected static final String SEPARATOR = "/";
  protected static final String PROP_NOTES = "description";
  protected static final String PROP_AUTHOR = "{" + Constants.NAMESPACE_CONTENT_MODEL + "}author";
  final String DEFAULT_DESCRIPTION="iDempiere Managed Space";

  protected String server_host = "";
  protected String server_port = "";
  protected String server_username = "";
  protected String server_password = "";
  /** Space Properties */
  protected String space_description = DEFAULT_DESCRIPTION;
  /** Content Properties */
  protected String content_description = "";
  protected String content_title = "";
  protected String content_author = "";

  protected final Store STORE = new Store(Constants.WORKSPACE_STORE, "SpacesStore");
  
  private static Logger log = Logger.getLogger(AlfrescoContentService.class.getCanonicalName());

  protected void setSessionProperties() {

    if (server_host.length() <= 0 || server_port.length() <= 0 || server_username.length() <= 0
        || server_password.length() <= 0) {
      Properties prop = new Properties();

      try {
        prop.load(getClass().getResourceAsStream("alfresco.properties"));
      } catch (Exception e) {
    	  log.info("Can not load alfresco.properties");
      }

      server_host = prop.getProperty("server.host");
      server_port = prop.getProperty("server.port");
      server_username = prop.getProperty("server.user");
      server_password = prop.getProperty("server.password");
    }
  }

  protected void startSession() {
    try {
      setSessionProperties();
      log.fine("Connecting to: " + "http://" + server_host + ":" + server_port + "/alfresco");
      WebServiceFactory.setEndpointAddress("http://" + server_host + ":" + server_port+ "/alfresco/api");
      AuthenticationUtils.startSession(server_username, server_password);
      AuthenticationUtils.getAuthenticationDetails();
    } catch (Exception e) {
    	log.severe("Can not initiate session with Alfresco server: "+e.getMessage());
    }
  }

  protected void endSession() {
	log.fine("Closing connection.");
    AuthenticationUtils.endSession();
  }

  protected NamedValue[] getContentProperties(String docname) {
    NamedValue[] properties = new NamedValue[] {
        Utils.createNamedValue(Constants.PROP_NAME, docname),
        Utils.createNamedValue(Constants.PROP_DESCRIPTION, content_description),
        Utils.createNamedValue(Constants.PROP_TITLE, content_title),
        Utils.createNamedValue(PROP_AUTHOR, content_author) };

    return properties;
  }

  protected NamedValue[] getSpaceProperties(String spacename) {
    NamedValue[] properties = new NamedValue[] {
        Utils.createNamedValue(Constants.PROP_DESCRIPTION, space_description),
        Utils.createNamedValue(Constants.PROP_NAME, spacename) };

    return properties;
  }

  protected RepositoryServiceSoapBindingStub getRepositoryService() {
    return WebServiceFactory.getRepositoryService();
  }

  protected AuthoringServiceSoapBindingStub getAuthoringService() {
    return WebServiceFactory.getAuthoringService();
  }

  protected ContentServiceSoapBindingStub getContentService() {
    return WebServiceFactory.getContentService();
  }

  protected ParentReference ReferenceToParent(Reference spaceref) {
    ParentReference parent = new ParentReference();

    parent.setStore(STORE);
    parent.setPath(spaceref.getPath());
    parent.setUuid(spaceref.getUuid());
    parent.setAssociationType(Constants.ASSOC_CONTAINS);

    return parent;
  }

  // Blanks are allowed in space names but not in paths. Because the path should depend on the name
  // we need a version of the name which contains no blanks

  protected String normilizeNodeName(String name) {
    return name.replace(" ", "_");
  }

  protected ParentReference getCompanyHome() {
    ParentReference companyHomeParent = new ParentReference(STORE, null, "/app:company_home",Constants.ASSOC_CONTAINS, null);
    return companyHomeParent;
  }

  protected Reference getSpaceReference(String path) {
    path = path.replaceAll(SEPARATOR, "/cm:");
    Reference spaceReference = new Reference(STORE, null, getCompanyHome().getPath()
        + normilizeNodeName(path));
    return spaceReference;
  }

  /** SPACES SECTION */
  protected Reference createSpace(String spacename) throws Exception {
    Reference space = null;

    // Create the space if it is not already existent
    try {

      // Therefore a reference to the maybe not existent space is required

      System.out.println("Entering space " + spacename);

      space = new Reference(STORE, null, getCompanyHome().getPath() + "/cm:"
          + normilizeNodeName(spacename));
      getRepositoryService().get(new Predicate(new Reference[] { space }, STORE, null));
    } catch (Exception e1) {
      System.out.println("The space named " + spacename + " does not exist. Creating it.");

      ParentReference companyHome = getCompanyHome();

      // Set Company Home as the parent space
      companyHome.setChildName(Constants.createQNameString(Constants.NAMESPACE_CONTENT_MODEL,
          normilizeNodeName(spacename)));

      // Set the space's property name
      NamedValue[] properties = getSpaceProperties(String.valueOf(spacename));

      // Create the space using CML (Content Manipulation Language)
      CMLCreate create = new CMLCreate("1", companyHome, null, null, null, Constants.TYPE_FOLDER,
          properties);
      CML cml = new CML();
      cml.setCreate(new CMLCreate[] { create });

      // Execute the CML create statement
      try {
        getRepositoryService().update(cml);
      } catch (Exception e2) {

        System.err.println("Can not create the space.");
        throw e2;
      }

    }

    return space;
  }

  protected Reference createSpace(Reference parentref, Object spacename) throws Exception {

    ParentReference parent = ReferenceToParent(parentref);
    Node node = null;
    try {
      System.out.println("Entering space " + spacename);
      // Check to see if the sample folder has already been created or not
      node = WebServiceFactory.getRepositoryService().get(
          new Predicate(new Reference[] { new Reference(STORE, null, parent.getPath()
              .concat("/cm:").concat(ISO9075.encode(String.valueOf(spacename)))) }, STORE, null))[0];
    } catch (Exception exception) {
      // Create parent reference to company home
      System.out.println("The space named " + spacename + " does not exist. Creating it.");
      ParentReference numericParentReference = new ParentReference(STORE, null, parent.getPath(),
          Constants.ASSOC_CONTAINS, Constants.createQNameString(Constants.NAMESPACE_CONTENT_MODEL,
              String.valueOf(spacename)));

      // Create folder
      NamedValue[] numericFolderProperties = getSpaceProperties(String.valueOf(spacename));

      CMLCreate numeriCreate = new CMLCreate("1", numericParentReference, null, null, null,
          Constants.TYPE_FOLDER, numericFolderProperties);
      CML numeriCml = new CML();
      numeriCml.setCreate(new CMLCreate[] { numeriCreate });
      UpdateResult[] results = WebServiceFactory.getRepositoryService().update(numeriCml);
      node = new Node(results[0].getDestination(), Constants.TYPE_FOLDER, null,
          numericFolderProperties);
    }
    return (node == null) ? null : node.getReference();
  }

  /** CONTENT SECTION */
  
  protected List<AlfrescoFile>  queryChildren(Reference parentref) throws Exception {
	  List<AlfrescoFile> ret=null;
	    RepositoryServiceSoapBindingStub repoService = getRepositoryService();
	    // Get a reference to the space we have named
	    
	    Predicate predicate = new Predicate(new Reference[] { new Reference(STORE, null, parentref.getPath()) }, STORE, null);
	    Node[] nodes = repoService.get(predicate);
	    // Create a query object, looking children
	    String strQueryChildren = "+PARENT:\"workspace://SpacesStore/"
	        + nodes[0].getReference().getUuid() + "\"";

	    Query queryChildrenSearch = new Query(Constants.QUERY_LANG_LUCENE, strQueryChildren);
	    System.out.println(">>> strQueryChildren: " + strQueryChildren);

	    QueryResult queryChildrenResult = repoService.queryChildren(nodes[0].getReference());
	    // Display the results
	    ResultSet resultSet = queryChildrenResult.getResultSet();
	    ResultSetRow[] rows = resultSet.getRows();
	    System.out.println(">>> Searching Children of \"@cm:name:\"" + parentref.getPath() + "\"");
	    ret = new ArrayList<AlfrescoFile>();
	    for (ResultSetRow row : rows) {
	      System.out.println("ID de Nodo: " + row.getNode().getId());
	      System.out.println("Tipo de Nodo: " + row.getNode().getType());
	      if (row.getNode().getType().endsWith("article")||row.getNode().getType().endsWith("content")) {
	        try {
	        	AlfrescoFile afile = new AlfrescoFile();
	        	
	        	String fileName="";
	        	for(NamedValue value : row.getColumns())
	        	{
	        		if(value.getName().endsWith("name"))
	        		{
	        			fileName=value.getValue();
	        			break;
	        		}
	        	}
	        	
	        	Reference refFile = new Reference(STORE, row.getNode().getId(), parentref.getPath().concat(
	                    "/cm:").concat(ISO9075.encode(String.valueOf(fileName))));
	        	
	        	afile.setFileName(fileName);
	        	afile.setPath(refFile.getPath());
	        	afile.setUuid(refFile.getUuid());
	        	afile.setPath(refFile.getPath());
	        	
	        	System.out.println("The version history:");
	            VersionHistory versionHistory = getAuthoringService().getVersionHistory(refFile);
	            if(versionHistory.getVersions()!=null && versionHistory.getVersions().length>0)
	            {
		            for (Version version : versionHistory.getVersions()) {
		            	afile.getVersionNumber().add(version.toString());
		            	afile.getVersionDesc().add(getVersionDetails(version).toString());
		            }
	            }
	        	afile.setFileData(getContentAsByte(refFile));
	        	ret.add(afile);
	          
	        } catch (Exception e) {
	          e.printStackTrace();

	        }
	      }
	    }
	    return ret;
}
  
  
  protected List<Reference> getContent(Reference parentref) throws Exception {
	  
	  List<Reference> ret=null;
	  
	  try {

	      Content[] read = getContentService().read(
	          new Predicate(new Reference[] { new Reference(STORE, null, parentref.getPath()) }, STORE, null),
	          Constants.PROP_CONTENT);
	      
	      ret = new ArrayList<Reference>();
	      for(Content content : read)
	      {
	    	  ret.add(content.getNode());
	      }
	      System.out.println("Got " + read.length + " content elements.");

	    } catch (Exception e) {
	      System.err.println("Can not get the content: " + e.getLocalizedMessage());
	      ret = null;
	    }
	  
	  return ret;
  }
  
  
  
  
  protected Reference getContent(Reference parentref, String docname) throws Exception {
    Reference content = null;

    try {

      Content[] read = getContentService().read(
          new Predicate(new Reference[] { new Reference(STORE, null, parentref.getPath().concat(
              "/cm:").concat(ISO9075.encode(String.valueOf(docname)))) }, STORE, null),
          Constants.PROP_CONTENT);
      content = read[0].getNode();
      System.out.println("Got " + read.length + " content elements.");

    } catch (Exception e) {
      System.err.println("Can not get the content: " + e.getLocalizedMessage());
      content = null;
    }

    return content;
  }
  
  protected Reference getDocumentReference(Reference parentref, String docname) throws Exception{
	  Reference doc = null;
	  try {
		  
		  doc = new Reference(STORE, parentref.getUuid(), parentref.getPath().concat(
                  "/cm:").concat(ISO9075.encode(String.valueOf(docname))));
		  
	} catch (Exception e) {
		System.err.println("Can not get the content: " + e.getLocalizedMessage());
		doc=null;
	}
	  
	  
	  return doc;
  }
  
  protected byte[] getContentAsByte(Reference currentref)throws Exception {
	    Content content = null;

	    try {

	      Content[] read = getContentService().read(
	          new Predicate(new Reference[] {currentref}, STORE, null), Constants.PROP_CONTENT);
	      content = read[0];
	      System.out.println("Got " + read.length + " content elements.");

	    } catch (Exception e) {
	      System.err.println("Can not get the content: " + e.getLocalizedMessage());
	      content = null;
	    }

	    ByteArrayOutputStream out = new ByteArrayOutputStream();
	    InputStream in = ContentUtils.getContentAsInputStream(content);
	    byte[] buf = new byte[2048];
	    int size;
	    while ((size = in.read(buf)) != -1) {
	      out.write(buf, 0, size);
	    }

	    return out.toByteArray();
  }
  

  protected byte[] getContentAsByte(Reference parentref, String docname) throws Exception {
    Content content = null;

    try {

      Content[] read = getContentService().read(
          new Predicate(new Reference[] { new Reference(STORE, null, parentref.getPath().concat(
              "/cm:").concat(ISO9075.encode(String.valueOf(docname)))) }, STORE, null),
          Constants.PROP_CONTENT);
      content = read[0];
      System.out.println("Got " + read.length + " content elements.");

    } catch (Exception e) {
      System.err.println("Can not get the content: " + e.getLocalizedMessage());
      content = null;
    }

    ByteArrayOutputStream out = new ByteArrayOutputStream();
    InputStream in = ContentUtils.getContentAsInputStream(content);
    byte[] buf = new byte[2048];
    int size;
    while ((size = in.read(buf)) != -1) {
      out.write(buf, 0, size);
    }

    return out.toByteArray();
  }

  protected Reference createDocument(Reference parentref, String docname, byte[] content)
      throws Exception {
    Reference document = null;
    ParentReference parent = ReferenceToParent(parentref);

    parent.setChildName(Constants.createQNameString(Constants.NAMESPACE_CONTENT_MODEL,
        normilizeNodeName(docname)));

    NamedValue[] properties = getContentProperties(docname);
    CMLCreate create = new CMLCreate("1", parent, null, null, null, Constants.TYPE_CONTENT,
        properties);
    CML cml = new CML();
    cml.setCreate(new CMLCreate[] { create });

    // Execute the CML create statement

    UpdateResult[] results = null;

    try {
      System.out.println("Creating the document " + docname);
      results = getRepositoryService().update(cml);
      document = results[0].getDestination();
    } catch (Exception e) {
      System.err.println("Can not create the document.");
      throw e;
    }

    // Set the content

    FileNameMap fileNameMap = URLConnection.getFileNameMap();
    String mime = fileNameMap.getContentTypeFor(docname);
    if (mime == null)
      mime = "application/octet-stream";

    ContentFormat format = new ContentFormat(mime, "UTF-8");

    try {
      System.out.println("Setting the content of the document");
      getContentService().write(document, Constants.PROP_CONTENT, content, format);
    } catch (Exception e2) {
      System.err.println("Can not set the content of the document.");
      throw e2;
    }

    return document;
  }

  protected Reference createDocument(Reference parentref, String docname, byte[] content,
      Aspect aspect) throws Exception {
    Reference document = null;
    ParentReference parent = ReferenceToParent(parentref);

    parent.setChildName(Constants.createQNameString(Constants.NAMESPACE_CONTENT_MODEL,
        normilizeNodeName(docname)));

    // CML for the Create
    NamedValue[] properties = getContentProperties(docname);
    CMLCreate create = new CMLCreate("1", parent, null, null, null, Constants.TYPE_CONTENT,
        properties);

    // CML for adding the Aspect
    String[] aspect_prop_tmp = aspect.getProperty_names_as_array();
    String[] aspect_val_tmp = aspect.getProperty_values_as_array();
    NamedValue[] aspect_properties = new NamedValue[aspect_prop_tmp.length];

    for (int i = 0; i < aspect_prop_tmp.length; i++) {
      aspect_properties[i] = Utils.createNamedValue(aspect.getAspect_domain() + aspect_prop_tmp[i],
          aspect_val_tmp[i]);
    }

    CMLAddAspect addaspect = new CMLAddAspect();
    addaspect.setAspect(aspect.getAspect_domain() + aspect.getAspect_name());
    addaspect.setProperty(aspect_properties);
    addaspect.setWhere(new Predicate(new Reference[] { parent }, null, null));
    addaspect.setWhere_id("1");

    // Set the CML
    CML cml = new CML();
    cml.setCreate(new CMLCreate[] { create });
    cml.setAddAspect(new CMLAddAspect[] { addaspect });

    // Execute the CML create statement
    UpdateResult[] results = null;

    try {
      System.out.println("Creating the document " + docname + " with aspect "
          + aspect.getAspect_name() + ".");
      results = getRepositoryService().update(cml);
      document = results[0].getDestination();
    } catch (Exception e) {
      System.err.println("Can not create the document: "+e.getMessage());
      throw e;
    }

    FileNameMap fileNameMap = URLConnection.getFileNameMap();
    String mime = fileNameMap.getContentTypeFor(docname);
    if (mime == null)
      mime = "application/octet-stream";
    // Set the content
    ContentFormat format = new ContentFormat(mime, "UTF-8");

    try {
      System.out.println("Setting the content of the document");
      getContentService().write(document, Constants.PROP_CONTENT, content, format);
    } catch (Exception e2) {
      System.err.println("Can not set the content of the document.");
      throw e2;
    }

    return document;

  }

  /** VERSIONING SECTION */
  protected Reference updateDocument(Reference ref, String docname, byte[] file_content, String description)
      throws Exception {

    Predicate itemToCheckOut = null;
    try {

      System.out.println("Versioning document");
      /*
       * Checkout the document to update, placing the working document in the same folder
       */
      itemToCheckOut = new Predicate(new Reference[] { ref }, STORE, null);

      /*
       * First confirm that the content reference does not have the versionable aspect applied
       */
      Node node = getRepositoryService().get(itemToCheckOut)[0];
      for (String aspect : node.getAspects()) {
        if (Constants.ASPECT_VERSIONABLE.equals(aspect) == false) {
          makeVersionable(itemToCheckOut);
        }
      }
      CheckoutResult checkOutResult = getAuthoringService().checkout(itemToCheckOut, null);

      /* Get a reference to the working copy */
      Reference workingCopyReference = checkOutResult.getWorkingCopies()[0];

      FileNameMap fileNameMap = URLConnection.getFileNameMap();
      String mime = fileNameMap.getContentTypeFor(docname);
      if (mime == null)
        mime = "application/octet-stream";
      // Set the content
      ContentFormat format = new ContentFormat(mime, "UTF-8");
      // Update the content of the working copy
      getContentService().write(workingCopyReference, Constants.PROP_CONTENT, file_content, format);

      // Now check the working copy in with a description of the change made that will be recorded
      // in the version history
      Predicate predicate = new Predicate(new Reference[] { workingCopyReference }, null, null);

      NamedValue[] comments = new NamedValue[] { Utils.createNamedValue(PROP_NOTES,
          description) };
      getAuthoringService().checkin(predicate, comments, false);

      itemToCheckOut = new Predicate(new Reference[] { ref }, STORE, null);
      updateProperties(itemToCheckOut, docname);
      // Get the version history
      System.out.println("The version history:");
      VersionHistory versionHistory = getAuthoringService().getVersionHistory(ref);
      for (Version version : versionHistory.getVersions()) {
        // Output the version details
        getVersionDetails(version);
      }
    } finally {

    }

    return itemToCheckOut.getNodes(0);
  }

  protected Reference updateDocument(Reference ref, String docname, byte[] file_content,
      Aspect localAspect) throws Exception {

    Predicate itemToCheckOut = null;
    try {
      System.out.println("Versioning document with Aspect: " + localAspect.getAspect_name());

      /*
       * Checkout the document to update, placing the working document in the same folder
       */
      itemToCheckOut = new Predicate(new Reference[] { ref }, STORE, null);

      /*
       * First confirm that the content reference does not have the versionable aspect applied
       */
      Node node = getRepositoryService().get(itemToCheckOut)[0];
      for (String aspect : node.getAspects()) {
        if (Constants.ASPECT_VERSIONABLE.equals(aspect) == false) {
          makeVersionable(itemToCheckOut);
        }
      }
      CheckoutResult checkOutResult = getAuthoringService().checkout(itemToCheckOut, null);

      /* Get a reference to the working copy */
      Reference workingCopyReference = checkOutResult.getWorkingCopies()[0];

      FileNameMap fileNameMap = URLConnection.getFileNameMap();
      String mime = fileNameMap.getContentTypeFor(docname);
      if (mime == null)
        mime = "application/octet-stream";
      // Set the content
      ContentFormat format = new ContentFormat(mime, "UTF-8");
      // Update the content of the working copy
      getContentService().write(workingCopyReference, Constants.PROP_CONTENT, file_content, format);

      // Now check the working copy in with a description of the change made that will be recorded
      // in the version history
      Predicate predicate = new Predicate(new Reference[] { workingCopyReference }, null, null);

      NamedValue[] comments = new NamedValue[] { Utils.createNamedValue(PROP_NOTES,
          "El contenido ha sido actualizado") };
      getAuthoringService().checkin(predicate, comments, false);

      // ADD Custom Aspect
      itemToCheckOut = new Predicate(new Reference[] { ref }, STORE, null);
      updateProperties(itemToCheckOut, docname);
      addCustomAspect(itemToCheckOut, localAspect);

      // Get the version history
      System.out.println("The version history:");
      VersionHistory versionHistory = getAuthoringService().getVersionHistory(ref);
      for (Version version : versionHistory.getVersions()) {
        // Output the version details
        getVersionDetails(version);
      }
    } finally {

    }

    return itemToCheckOut.getNodes(0);
  }

  private UpdateResult[] updateProperties(Predicate itemToUpdate, String docname) throws Exception {
    // create CMLUpdate instances to update corresponding properties.
    CMLUpdate updateGlobalProperties = new CMLUpdate(getContentProperties(docname), itemToUpdate,
        null);

    // create a CML instance and execute updates.
    CML cml = new CML();
    cml.setUpdate(new CMLUpdate[] { updateGlobalProperties });
    UpdateResult[] result = getRepositoryService().update(cml);

    System.out.println("properties successfully updated...");
    return result;
  }

  private void makeVersionable(Predicate itemToCheckOut) throws Exception {

    try {
      /* Create the add aspect query object */
      CMLAddAspect addAspect = new CMLAddAspect(Constants.ASPECT_VERSIONABLE, null, itemToCheckOut,
          null);

      /* Create the content management language query */
      CML cml = new CML();
      cml.setAddAspect(new CMLAddAspect[] { addAspect });

      getRepositoryService().update(cml);

    } catch (RepositoryFault e) {
      throw e;
    } catch (RemoteException e) {
      throw e;
    }
  }

  private void addCustomAspect(Predicate itemToCheckOut, Aspect aspect) throws Exception {

    try {
      // CML for adding the Aspect
      String[] aspect_prop_tmp = aspect.getProperty_names_as_array();
      String[] aspect_val_tmp = aspect.getProperty_values_as_array();
      NamedValue[] aspect_properties = new NamedValue[aspect_prop_tmp.length];

      for (int i = 0; i < aspect_prop_tmp.length; i++) {
        aspect_properties[i] = Utils.createNamedValue(aspect.getAspect_domain()
            + aspect_prop_tmp[i], aspect_val_tmp[i]);
      }

      CMLAddAspect addaspect = new CMLAddAspect();
      addaspect.setAspect(aspect.getAspect_domain() + aspect.getAspect_name());
      addaspect.setProperty(aspect_properties);
      addaspect.setWhere(itemToCheckOut);

      /* Create the content management language query */
      CML cml = new CML();
      cml.setAddAspect(new CMLAddAspect[] { addaspect });

      getRepositoryService().update(cml);

    } catch (RepositoryFault e) {
      throw e;
    } catch (RemoteException e) {
      throw e;
    }
  }

  
  protected void deleteSpace(Reference space) throws Exception 
  {
  		CMLDelete delete = new CMLDelete(new Predicate(new Reference[]{space},null,null));
  		CML cml = new CML();
  		cml.setDelete(new CMLDelete[]{delete});
  		//Execute the CMLDelete statement
         try {
         	 	System.out.println("Deleting the space " + space.getPath());
         	 	WebServiceFactory.getRepositoryService().update(cml);
          } catch (Exception e2) {
  				System.err.println("Can not delete the space.");
  				throw e2;
          }     
  }
  
  
  protected Reference deleteSpace(AlfrescoFile alfFile) throws Exception{
	  Reference doc = null;
	  
	  doc = new Reference(STORE, alfFile.getUuid(), alfFile.getPath());
	  
	  Predicate predicate = new Predicate(new Reference[] {doc}, STORE,null);
	
	  CMLDelete delete = new CMLDelete(predicate);
	  CML cml = new CML();
	  cml.setDelete(new CMLDelete[]{delete});
	  //Execute the CMLDelete statement
       try {
       	 	System.out.println("Deleting the document/space " + doc.getPath());
       	 	WebServiceFactory.getRepositoryService().update(cml);
        } catch (Exception e2) {
				System.err.println("Can not delete the space.");
				throw e2;
        }
	  
	  return doc;
  }
  
  
  private String[] getVersionDetails(Version version) {
    String[] versionLabelAndDesc = new String[2];

    String versionLabel = null;
    String description = null;
    for (NamedValue namedValue : version.getCommentaries()) {
      if (namedValue.getName().equals(PROP_NOTES) == true) {
        description = namedValue.getValue();
      }
    }

    versionLabel = version.getLabel();
    versionLabelAndDesc[0] = versionLabel;
    versionLabelAndDesc[1] = description;
    System.out.println("Version: " + versionLabel + " - Description:" + description);
    return versionLabelAndDesc;
  }
  
  protected List<String[]> getProperties(Reference ref){
	 
	List<String[]> props = new ArrayList<String[]>();
	  
	String[] prop = new String[]{};
	
    try {
    	Predicate itemToCheckOut = new Predicate(new Reference[] { ref }, STORE, null);
		Node node = getRepositoryService().get(itemToCheckOut)[0];
		for(NamedValue nval : node.getProperties()){
			if(!nval.getIsMultiValue()){
				String name = "Undefined";
				try{
				name = nval.getName().substring(nval.getName().lastIndexOf("}")+1);
				}catch (Exception e) {}
				if(!name.equals("content")){
					prop = new String[]{name,nval.getValue()};
					props.add(prop);
				}
			}
		}
	} catch (RepositoryFault e1) {
		System.out.println("No data");
	} catch (RemoteException e1) {
		System.out.println("No data");
	}
	
	
	
	VersionHistory versionHistory;
	try {
		versionHistory = getAuthoringService().getVersionHistory(ref);
		if (versionHistory != null && versionHistory.getVersions()!=null)
		  {
			String title = "Versioned Document";
			String values = "";
			 for (Version version : versionHistory.getVersions())
			 {
				 String[] verPorp = getVersionDetails(version);
				 values = values + "Version:"+verPorp[0]+" / ";
				 values = values + "Description:"+verPorp[1]+" / ";
				 values = values + " -- ";
			}
			 prop = new String[]{title,values};
			 props.add(prop);
		 }
	} catch (AuthoringFault e) {
		System.out.println("No versions");
	} catch (RemoteException e) {
		System.out.println("No versions");
	}
	  
	return props;
	  
  }

}
